import { system, world, ItemStack } from "@minecraft/server";

function updateQuiverLore(quiver, arrowCount, maxCapacity) {
    let lore = quiver.getLore() ?? [];
    let updated = false;
    for (let j = 0; j < lore.length; j++) {
        if (lore[j].includes('Arrows:')) {
            lore[j] = `§r§7Arrows: ${arrowCount}/${maxCapacity}`;
            updated = true;
            break;
        }
    }
    if (!updated) {
        lore.push(`§r§7Arrows: ${arrowCount}/${maxCapacity}`);
    }
    quiver.setLore(lore);
}

function handleQuiverArrowChange(player, quiver, equip, arrowChange, maxCapacity) {
    const currentArrowCount = quiver.getDynamicProperty('arrows') ?? 0;
    const newArrowCount = currentArrowCount + arrowChange;

    quiver.setDynamicProperty('arrows', newArrowCount);
    updateQuiverLore(quiver, newArrowCount, maxCapacity);
    equip.setEquipment('Offhand', quiver);
}

function transferArrowsToQuiver(player, quiver, maxCapacity) {
    const inventory = player.getComponent("inventory").container;
    let totalArrows = 0;
    const arrow = new ItemStack("minecraft:arrow");

    for (let i = 0; i < inventory.size; i++) {
        const checkSlot = inventory.getItem(i);
        if (checkSlot && checkSlot.isStackableWith(arrow)) {
            totalArrows += checkSlot.amount;
        }
    }

    const currentArrowCount = quiver.getDynamicProperty('arrows') ?? 0;

    if (totalArrows > 0 && currentArrowCount < maxCapacity) {
        const arrowsToTransfer = Math.min(totalArrows, maxCapacity - currentArrowCount);
        player.runCommandAsync(`clear @s minecraft:arrow 0 ${arrowsToTransfer}`);
        quiver.setDynamicProperty('arrows', currentArrowCount + arrowsToTransfer);
        updateQuiverLore(quiver, currentArrowCount + arrowsToTransfer, maxCapacity);
    }
}

system.runInterval(() => {
    for (const player of world.getPlayers()) {
        const inventory = player.getComponent("inventory").container;
        const equip = player.getComponent('equippable');
        const offhand = equip.getEquipment('Offhand');

        if (!offhand) continue;

        const quiverType = offhand.typeId;
        let maxCapacity = 0;

        switch (quiverType) {
            case "fiery:quiver":
                maxCapacity = 192;
                break;
            case "fiery:iron_quiver":
                maxCapacity = 256;
                break;
            case "fiery:diamond_quiver":
                maxCapacity = 384;
                break;
            case "fiery:netherite_quiver":
                maxCapacity = 512;
                break;
            default:
                continue;
        }

        let totalArrows = 0;
        const arrow = new ItemStack("minecraft:arrow");
        for (let i = 0; i < inventory.size; i++) {
            const checkSlot = inventory.getItem(i);
            if (checkSlot && checkSlot.isStackableWith(arrow)) {
                totalArrows += checkSlot.amount;
            }
        }

        const currentArrowCount = offhand.getDynamicProperty('arrows') ?? 0;

        if ((totalArrows == 1 || totalArrows == 0) && (totalArrows == 1 || inventory.emptySlotsCount !== 0) && currentArrowCount > 0) {
            inventory.addItem(new ItemStack("minecraft:arrow", 1));
            handleQuiverArrowChange(player, offhand, equip, -1, maxCapacity);
            continue;
        }

        if (totalArrows > 2 && currentArrowCount < maxCapacity) {
            for (let i = 0; i < inventory.size; i++) {
                const slot = inventory.getItem(i);
                if (!slot || slot.typeId !== "minecraft:arrow") continue;

                player.runCommandAsync(`clear @s minecraft:arrow 0 1`);
                handleQuiverArrowChange(player, offhand, equip, 1, maxCapacity);
                break;
            }
        }
    }
}, 5);

world.afterEvents.itemUse.subscribe((ev) => {
    const player = ev.source;
    if (!player) return;

    const equip = player.getComponent('equippable');
    const mainhand = equip.getEquipment('Mainhand');
    const inventory = player.getComponent("inventory").container;

    if (!mainhand) return;

    const quiverType = mainhand.typeId;
    let maxCapacity = 0;

    switch (quiverType) {
        case "fiery:quiver":
            maxCapacity = 192;
            break;
        case "fiery:iron_quiver":
            maxCapacity = 256;
            break;
        case "fiery:diamond_quiver":
            maxCapacity = 384;
            break;
        case "fiery:netherite_quiver":
            maxCapacity = 512;
            break;
        default:
            return;
    }

    if (["fiery:quiver", "fiery:iron_quiver", "fiery:diamond_quiver", "fiery:netherite_quiver"].includes(quiverType)) {
        const currentArrowCount = mainhand.getDynamicProperty('arrows') ?? 0;

        if (currentArrowCount > 0) {
            for (let i = 0; i < currentArrowCount; i++) {
                inventory.addItem(new ItemStack("minecraft:arrow", 1));
            }
            player.runCommandAsync(`playsound armor.equip_generic @a[r=16] ~~~`);

            mainhand.setDynamicProperty('arrows', 0);
            updateQuiverLore(mainhand, 0, maxCapacity);
            equip.setEquipment('Mainhand', mainhand);
        } else {
            player.runCommandAsync(`playsound armor.equip_generic @a[r=16] ~~~`);
            transferArrowsToQuiver(player, mainhand, maxCapacity);
            equip.setEquipment('Mainhand', mainhand);  // Ensure the mainhand is updated in equipment
        }
    }
});
